"""OntoRuler class for running Spacy."""
import multiprocessing
import os
import pickle
from pathlib import Path

import pandas as pd
import spacy
from scispacy.linking import EntityLinker  # noqa F401
from spacy.language import Language  # noqa F401
from spacy.matcher import PhraseMatcher
from spacy.pipeline import entityruler  # noqa F401
from spacy.tokens import Doc, Span

from ontorunner import (DATA_DIR, ONTO_TERMS_FILENAME,
                        PATTERN_LIST_PICKLED_FILENAME,
                        PHRASE_MATCHER_PICKLED_FILENAME, SERIAL_DIR_NAME,
                        SETTINGS_FILE_PATH, TERMS_DIR_NAME,
                        TERMS_PICKLED_FILENAME, _get_config)


class OntoRuler(object):
    """OntoRuler class."""

    def __init__(
        self,
        data_dir: Path = DATA_DIR,
        settings_filepath: Path = SETTINGS_FILE_PATH,
        linker: str = "umls",  # Must be one of 'umls' or 'mesh'.
        to_pickle: bool = True,
    ):
        # self.parent_dir = parent_dir
        self.data_dir = data_dir
        self.serial_dir = os.path.join(self.data_dir, SERIAL_DIR_NAME)
        self.terms_dir = os.path.join(self.data_dir, TERMS_DIR_NAME)

        self.phrase_matcher_pickled = os.path.join(
            self.serial_dir, PHRASE_MATCHER_PICKLED_FILENAME
        )
        self.terms_pickled = os.path.join(self.serial_dir, TERMS_PICKLED_FILENAME)
        self.pattern_list_pickled = os.path.join(
            self.serial_dir, PATTERN_LIST_PICKLED_FILENAME
        )
        self.phrase_matcher_pickled = os.path.join(
            self.serial_dir, PHRASE_MATCHER_PICKLED_FILENAME
        )
        self.settings_file = settings_filepath
        self.combined_onto_file = os.path.join(self.terms_dir, ONTO_TERMS_FILENAME)
        self.combined_onto_pickle = os.path.join(
            self.serial_dir, (ONTO_TERMS_FILENAME + ".pickle")
        )
        self.label = "ontology"
        self.phrase_matcher_attr = "LOWER"
        self.multiprocessing = False
        # False -> Use single processor;
        # True -> Use multiple processors
        self.processing_threshold = 100_000
        self.terms = {}
        self.list_of_pattern_dicts = []
        self.list_of_doc_obj = []
        self.nlp = spacy.load("en_ner_craft_md")
        self.nlp.rename_pipe("ner", "craft_ner")  # To avoid conflict
        # Source for below: https://spacy.io/usage/processing-pipelines
        self.nlp.add_pipe(
            "ner", source=spacy.load("en_core_web_sm"), before="craft_ner"
        )

        if os.path.isfile(self.phrase_matcher_pickled):
            print("Found serialized files!")
            with open(os.path.join(self.terms_pickled), "rb") as tf:
                self.terms = pickle.load(tf)
            with open(self.pattern_list_pickled, "rb") as plp:
                self.list_of_pattern_dicts = pickle.load(plp)
            with open(self.phrase_matcher_pickled, "rb") as pmp:
                self.phrase_matcher = pickle.load(pmp)
            print("Serialized files imported!")

        else:
            self.extract_termlist_info(to_pickle=to_pickle)

        ruler = self.nlp.add_pipe("entity_ruler", after="craft_ner")
        ruler.add_patterns(self.list_of_pattern_dicts)
        print("Patterns added!")

        # variables for spans and docs extensions
        self.span_term_extension = "is_an_ontology_term"
        self.span_id_extension = "object_id"
        self.has_id_extension = "has_curies"

        Span.set_extension(self.span_term_extension, default=False, force=True)
        Span.set_extension(self.span_id_extension, default=False, force=True)
        Span.set_extension("object_category", default=False, force=True)
        Span.set_extension("object_label", default=False, force=True)
        Span.set_extension("object_match_field", default=False, force=True)
        Span.set_extension("origin", default=False, force=True)
        Span.set_extension("start", default=False, force=True)
        Span.set_extension("end", default=False, force=True)

        Span.set_extension(self.has_id_extension, getter=self.has_curies, force=True)

        Doc.set_extension(self.has_id_extension, getter=self.has_curies, force=True)
        Doc.set_extension(self.label.lower(), default=[], force=True)
        print("Extensions set!")

        self.nlp.add_pipe(
            "scispacy_linker",
            config={"resolve_abbreviations": True, "linker_name": linker},
        )
        print("SciSpacy loaded!")

    # getter function for doc level
    def has_curies(self, tokens):
        """Check if any one token has CURIEs

        :param tokens: token
        :return: Boolean
        """
        return any([t._.get(self.span_term_extension) for t in tokens])

    def get_ont_terms_df(self, to_pickle) -> pd.DataFrame:
        """Get Ontology terms from external source in the form of a pandas DataFrame.

        :return: Pandas DataFrame for of termlist.
        """
        cols = [
            "CUI",
            "origin",
            "CURIE",
            "matched_term",
            "description",
            "object_category",
        ]

        if not os.path.isfile(self.combined_onto_pickle):
            if not os.path.isfile(self.combined_onto_file):
                termlist = _get_config("termlist", self.settings_file)
                df = pd.concat(
                    [
                        pd.read_csv(
                            os.path.join(self.terms_dir, f),
                            sep="\t",
                            low_memory=False,
                            header=None,
                        )
                        for f in termlist
                    ]
                )
                df = df.drop_duplicates()
                df.to_csv(self.combined_onto_file, sep="\t", index=None, header=False)
                df.to_pickle(self.combined_onto_pickle)
            else:
                df = pd.read_csv(self.combined_onto_file, sep="\t", low_memory=False)
                if to_pickle:
                    df.to_pickle(self.combined_onto_pickle)
        else:
            df = pd.read_pickle(self.combined_onto_pickle)
        df.columns = cols
        df = df.drop(["CUI"], axis=1)
        df = df.fillna("")
        return df

    def get_terms_patterns(self, *args):
        """Get dictionaries of terms and patterns,
        along with doc format of the term.

        :return: Dictionary of terms;
        Dictionary of patterns;
        doc object of the term
        """
        origin, object_id, matched_term, description, object_category = args[0]
        terms_dict = {}
        pattern_dict = {}
        object_match_field = ""

        if "[SYNONYM_OF:" in description:
            object_label = description.split("[SYNONYM_OF:")[-1].rstrip("]")
            object_match_field = "hasRelatedSynonym"
        else:
            object_label = matched_term
            # object_match_field = "isExactMatch"

        if matched_term is not None and matched_term == matched_term:
            terms_dict[matched_term.lower()] = {
                "object_id": object_id,
                "object_category": object_category,
                "object_label": object_label,
                "object_match_field": object_match_field,
                "origin": origin,
            }
            pattern_dict["id"] = object_id
            pattern_dict["label"] = origin.split(".")[
                0
            ]  # could be object_id/object_label
            pattern_dict["pattern"] = matched_term

        return terms_dict, pattern_dict, self.nlp(matched_term)

    def extract_termlist_info(self, to_pickle: bool):
        df = self.get_ont_terms_df(to_pickle=to_pickle)

        if len(df) > self.processing_threshold:
            number_of_processes = multiprocessing.cpu_count() // 2 - 1
            if number_of_processes <= 1:
                self.multiprocessing = False
            else:
                self.multiprocessing = True

        # iterate over terms in ontology
        if self.multiprocessing:
            # * Multiprocessing ***********************************************
            with multiprocessing.Pool(processes=number_of_processes) as pool:
                results = pool.map(self.get_terms_patterns, df.to_records(index=False))

            self.terms = {
                k: v for d in [result[0] for result in results] for k, v in d.items()
            }
            self.list_of_pattern_dicts = [result[1] for result in results]
            self.list_of_doc_obj = [result[2] for result in results]

        else:
            # * Single process **********************************************
            for (
                origin,
                object_id,
                matched_term,
                description,
                object_category,
            ) in df.to_records(index=False):
                terms, patterns, object_doc = self.get_terms_patterns(
                    (
                        origin,
                        object_id,
                        matched_term,
                        description,
                        object_category,
                    )
                )
                self.terms.update(terms)
                self.list_of_pattern_dicts.append(patterns)
                self.list_of_doc_obj.append(object_doc)
            # ***************************************************************

        self.phrase_matcher = PhraseMatcher(
            self.nlp.vocab, attr=self.phrase_matcher_attr
        )
        self.phrase_matcher.add(self.label, None, *self.list_of_doc_obj)

        if to_pickle:
            # Dump serialized files
            # self.nlp.to_disk(CUSTOM_PIPE_DIR)
            with open(self.terms_pickled, "wb") as tp:
                pickle.dump(self.terms, tp)
            with open(self.pattern_list_pickled, "wb") as plp:
                pickle.dump(self.list_of_pattern_dicts, plp)
            # with open(OBJ_DOC_LIST_PICKLED, "wb") as odlp:
            #     pickle.dump(self.list_of_doc_obj, odlp)
            with open(self.phrase_matcher_pickled, "wb") as pmp:
                pickle.dump(self.phrase_matcher, pmp)

            print("Serialized files dumped!")



"""Post process the NER results."""
import os

from ontorunner import DATA_DIR

NODE_AND_EDGE_NAME = "nodes_and_edges"

NODE_AND_EDGE_DIR = os.path.join(DATA_DIR, NODE_AND_EDGE_NAME)
SUBCLASS_PREDICATE = "biolink:subclass_of"
SUBCLASS_RELATION = "rdfs:subClassOf"



import os
import shutil

import click
from genericpath import isdir
from kgx.cli.cli_utils import transform

from ontorunner import DATA_DIR_NAME, PARENT_DIR, SERIAL_DIR_NAME
from ontorunner.converters import biohub_converter as bc

SERIAL_DIR = os.path.join(PARENT_DIR, DATA_DIR_NAME, SERIAL_DIR_NAME)


def json2tsv(input, output) -> None:
    """
    Converts an JSON file into 'nodes' and 'edges' TSV.

    :param input: Input file (JSON file).
    :param ouput: Output file name desired.
    :return: None.
    """
    if input:
        if output is None:
            output = "data/nodes_and_edges/"

        transform(
            inputs=[input],
            input_format="obojson",
            output=output,
            output_format="tsv",
        )
    else:
        input_folder = "data/input/"
        output_folder = "data/nodes_and_edges/"

        for subdir, dirs, files in os.walk(input_folder):
            for file in files:
                fn, ext = os.path.splitext(file)

                if ext == ".json":
                    transform(
                        inputs=[subdir + file],
                        input_format="obojson",
                        output=output_folder + fn,
                        output_format="tsv",
                    )


def prepare_termlist(input, output) -> None:
    """
    Generates a Bio Term Hub formatted term list for use with OGER.

    :param input: Input file 'ontology_nodes.tsv'.
    :param ouput: TSV file of list of terms 'ontology_termlist.tsv'.
    :return: None.
    """
    bc.parse(input, output)


@click.group()
def cli():
    pass


@cli.command("json2tsv")
@click.option("--input", "-i", type=click.Path(exists=True))
@click.option("--output", "-o", type=str)
def json2tsv_click(input, output):
    json2tsv(input, output)


@cli.command("prepare-termlist")
@click.option("--input", "-i", type=click.Path(exists=True), required=True)
@click.option("--output", "-o", type=str, required=True)
def prepare_termlist_click(input, output):
    prepare_termlist(input, output)


@cli.command("delete-cache")
def delete_cache():
    for f in os.listdir(SERIAL_DIR):
        path = os.path.join(SERIAL_DIR, f)
        if isdir(path):
            shutil.rmtree(path)
        else:
            os.remove(path)
    print("Serialzed data folder purged!")


if __name__ == "__main__":
    cli()



"""Run Spacy."""
import os
from glob import glob
from multiprocessing import freeze_support
from pathlib import Path

import click
import pandas as pd
from spacy.tokens import Doc, Span

from ontorunner import (DATA_DIR, INPUT_DIR_NAME, OUTPUT_DIR_NAME,
                        SETTINGS_FILE_PATH, _get_config)
from ontorunner.pipes.OntoRuler import OntoRuler
from ontorunner.post import NODE_AND_EDGE_NAME, util

SCI_SPACY_LINKERS = ["umls", "mesh"]


def get_token_info(doc: Doc) -> pd.DataFrame:
    """Get metadata associated with spans within a document.

    :param doc: Doc object.
    :return: Pandas DataFrame.
    """ """"""
    key_list = [
        "matched_term",
        "POS",
        "tag",
        "scispacy_object_category",
        "object_id",
        "object_category",
        "object_label",
        "object_match_field",
        "origin",
        "sentence",
        "start",
        "end",
    ]

    # # Filter df to remove certain POS'
    # """
    # List of POS codes
    # POS | DESCRIPTION | EXAMPLES
    # ADJ | adjective | *big, old, green, incomprehensible, first*
    # ADP | adposition | *in, to, during*
    # ADV | adverb | *very, tomorrow, down, where, there*
    # AUX | auxiliary | *is, has (done), will (do), should (do)*
    # CONJ | conjunction | *and, or, but*
    # CCONJ | coordinating conjunction | *and, or, but*
    # DET | determiner | *a, an, the*
    # INTJ | interjection | *psst, ouch, bravo, hello*
    # NOUN | noun | *girl, cat, tree, air, beauty*
    # NUM | numeral | *1, 2017, one, seventy-seven, IV, MMXIV*
    # PART | particle | *’s, not,*
    # PRON | pronoun | *I, you, he, she, myself, themselves, somebody*
    # PROPN | proper noun | *Mary, John, London, NATO, HBO*
    # PUNCT | punctuation | *., (, ), ?*
    # SCONJ | subordinating conjunction | *if, while, that*
    # SYM | symbol | *$, %, §, ©, +, −, ×, ÷, =, :), 😝*
    # VERB | verb | *run, runs, running, eat, ate, eating*
    # X | other | *sfpksdpsxmsa*
    # SPACE | space

    # """
    ignore_pos = [
        "ADP",
        "CCONJ",
        "CONJ",
        "DET",
        "INTJ",
        "SCONJ",
        "PART",
        "PUNCT",
        "PRON",
        "AUX",
        "NUM",
        "ADV",
        "VERB",
    ]
    onto_dict = {}
    for k in key_list:
        onto_dict[k] = []

    unwanted_labels = [
        "ORG",
        "GPE",
        "LOC",
    ]
    unwanted_span_list = []
    valid_span = True
    df = pd.DataFrame()

    for span in doc.ents:
        # Filter out spans with labels that are irrelevant
        if span.label_ in unwanted_labels:
            unwanted_span_list.append(span.text)
        else:
            if span._.is_an_ontology_term and span.text not in unwanted_span_list:
                valid_span = any([token.pos_ not in ignore_pos for token in span])

                if valid_span:
                    pos_list = ", ".join([token.pos_ for token in span])
                    tag_list = ", ".join([token.tag_ for token in span])
                    onto_dict["matched_term"].append(span.text)
                    onto_dict["POS"].append(pos_list)
                    onto_dict["tag"].append(tag_list)
                    onto_dict["scispacy_object_category"].append(span.label_)
                    onto_dict["object_id"].append(span._.object_id)
                    onto_dict["object_category"].append(span._.object_category)
                    onto_dict["object_label"].append(span._.object_label)
                    onto_dict["object_match_field"].append(span._.object_match_field)
                    onto_dict["origin"].append(span._.origin)
                    onto_dict["sentence"].append(span.sent)
                    onto_dict["start"].append(span.start_char)
                    onto_dict["end"].append(span.end_char)
                df = pd.DataFrame.from_dict(onto_dict)
                # Filter out terms that may have been
                # missed by the 'valid_span' flag determination.
                df = df[~df["matched_term"].isin(unwanted_span_list)]

    return df


def explode_df(df: pd.DataFrame) -> pd.DataFrame:
    """Explode multiple DataFrames in a single row into multiple rows.

    :param df: Dataframe to be exploded.
    :return: Exploded DataFrame where each row correspond to a row in the DataFrame.
    """
    new_df = pd.DataFrame()
    for _, doc_obj_row in df.iterrows():
        tmp_df = doc_obj_row[1]
        tmp_df.insert(0, "id", doc_obj_row.id)
        new_df = pd.concat([new_df, tmp_df], ignore_index=True)

    new_df = new_df.rename(columns={"id": "document_id"})
    return new_df


def onto_tokenize(doc: Doc, onto_ruler_obj: OntoRuler) -> Doc:
    """Set custom span information from the Doc object.

    :param doc: Doc object.
    :param onto_ruler_obj: OntoRuler object.
    :return: Doc object.
    """
    # matches = onto_ruler_obj.match(doc)
    matches = onto_ruler_obj.phrase_matcher(doc)
    spans = [
        Span(doc, start, end, label=onto_ruler_obj.label)
        for match_id, start, end in matches
    ]
    # doc.spans[onto_ruler_obj.label] = spans

    for _, span in enumerate(spans):
        span._.set("has_curies", True)

        if span.text.lower() in onto_ruler_obj.terms.keys():
            span._.set(onto_ruler_obj.span_term_extension, True)
            span._.set(
                "object_id",
                onto_ruler_obj.terms[span.text.lower()]["object_id"],
            )
            span._.set(
                "object_category",
                onto_ruler_obj.terms[span.text.lower()]["object_category"],
            )
            span._.set(
                "object_label",
                onto_ruler_obj.terms[span.text.lower()]["object_label"],
            )
            span._.set(
                "object_match_field",
                onto_ruler_obj.terms[span.text.lower()]["object_match_field"],
            )
            span._.set("origin", onto_ruler_obj.terms[span.text.lower()]["origin"])

    return doc


def get_knowledge_base_enitities(doc: Doc, onto_ruler_obj: OntoRuler) -> pd.DataFrame:
    """Get information from the SciSpacy pipeline.

    :param doc: Doc object.
    :param onto_ruler_obj: OntoRuler object.
    :return: Pandas DataFrame.
    """
    linker = onto_ruler_obj.nlp.get_pipe("scispacy_linker")
    ent_dict = {}
    key_list = ["cui", "matched_term", "aliases", "definition", "tui"]
    for k in key_list:
        ent_dict[k] = []

    for entity in doc.ents:
        for kb_ent in entity._.kb_ents:
            ent_object = linker.kb.cui_to_entity[kb_ent[0]]
            ent_dict["cui"].append(ent_object.concept_id)
            ent_dict["matched_term"].append(ent_object.canonical_name)
            ent_dict["aliases"].append(ent_object.aliases)
            ent_dict["definition"].append(ent_object.definition)
            ent_dict["tui"].append(ent_object.types)

    return pd.DataFrame.from_dict(ent_dict)


def export_tsv(df: pd.DataFrame, data_dir: str, fn: str) -> None:
    """Export pandas DataFrame object into a TSV file.

    :param df: Pandas DataFrame.
    :param data_dir: Destination directory for export.
    :param fn: Filename.
    """
    fn_path = os.path.join(data_dir, OUTPUT_DIR_NAME, fn + ".tsv")
    df.to_csv(fn_path, sep="\t", index=None)


@click.group()
def main():
    """
    Spacy module to run NER.

    e.g. poetry run python -m ontorunner.spacy_module run-spacy
    """
    pass


def run_spacy(
    data_dir: Path = DATA_DIR,
    settings_file: Path = SETTINGS_FILE_PATH,
    linker: str = "umls",
    to_pickle: bool = True,
    need_ancestors: bool = True,
):
    """
    Run spacy with sciSpacy pipeline.

    :param data_dir: Path to the data directory.
    :param settings: Path to settings.ini file.
    :param linker: Type of sciSpacy linker desired ([umls]/mesh).
    """
    if linker not in SCI_SPACY_LINKERS:
        raise (
            ValueError(
                f"SciSpacy linker provided '{linker}' is invalid."
                f"Choose one of the following: {SCI_SPACY_LINKERS}"
            )
        )
    onto_ruler_obj = OntoRuler(
        data_dir=data_dir,
        settings_filepath=settings_file,
        linker=linker,
        to_pickle=to_pickle,
    )
    batch_size = 10000
    input_dir_path = os.path.join(data_dir, INPUT_DIR_NAME) + "/*.tsv"
    input_file_list = glob(input_dir_path)
    list_of_input_dfs = []
    for fn in input_file_list:
        in_df = pd.read_csv(fn, sep="\t", low_memory=False)
        list_of_input_dfs.append(in_df)

    input_df = pd.concat(list_of_input_dfs, axis=0, ignore_index=True)

    input_df["spacy_doc"] = list(
        onto_ruler_obj.nlp.pipe(input_df["text"].values, batch_size=batch_size)
    )

    input_df["spacy_doc_tok"] = input_df["spacy_doc"].apply(
        lambda row: onto_tokenize(row, onto_ruler_obj)
    )

    input_df["spacy_tokens"] = input_df["spacy_doc_tok"].apply(
        lambda row: get_token_info(row)
    )

    input_df["spacy_kb_ent"] = input_df["spacy_doc_tok"].apply(
        lambda row: get_knowledge_base_enitities(row, onto_ruler_obj)
    )

    kb_df = explode_df(input_df[["id", "spacy_kb_ent"]])
    onto_df = explode_df(input_df[["id", "spacy_tokens"]])
    # nlp_df = doc_to_df(dframcy, input_df[["id", "spacy_doc"]])

    stopwords_file_path = os.path.join(data_dir, _get_config("termlist_stopwords")[0])
    stopwords_file = open(stopwords_file_path, "r")
    stopwords = stopwords_file.read().splitlines()
    onto_df = onto_df.loc[~onto_df["matched_term"].isin(stopwords)]

    onto_df = util.consolidate_rows(onto_df)
    onto_df = util.get_column_doc_ratio(onto_df, "object_label")
    onto_df = util.get_column_doc_ratio(onto_df, "matched_term")
    if need_ancestors:
        nodes_and_edges_dir = os.path.join(data_dir, NODE_AND_EDGE_NAME)
        onto_df = util.get_ancestors(
            df=onto_df, nodes_and_edges_dir=nodes_and_edges_dir
        )

    onto_df = onto_df.astype(str).drop_duplicates()
    kb_df = kb_df.astype(str).drop_duplicates()

    export_tsv(kb_df, data_dir, "sciSpacy_" + linker + "_ontoRunNER")
    export_tsv(onto_df, data_dir, "ontology_ontoRunNER")


@main.command("run-spacy")
@click.option("-d", "--data-dir", help="Data directory path.", default=DATA_DIR)
@click.option(
    "-s",
    "--settings-file",
    help="settings.ini file path.",
    default=SETTINGS_FILE_PATH,
)
@click.option(
    "-l",
    "--linker",
    type=click.Choice(["umls", "mesh"]),
    help="Which sciSpacy linker to use.('umls'[Default] or 'mesh')",
    default="umls",
)
@click.option(
    "-p",
    "--pickle-files",
    help="Boolean to determine if intermediate files should be pickled or no",
    default=True,
)
@click.option("--need-ancestors", "-a", type=bool, default=False)
def run_spacy_click(
    data_dir: Path,
    settings_file: Path,
    linker: str,
    pickle_files: bool,
    need_ancestors: bool,
):
    """CLI for running the spacy module.

    :param data_dir: Data dorectory path.
    :param settings_file: Filepath for settings.ini file.
    :param linker: Type of sciSpacy linker desired ([umls]/mesh).
    :param pickle_files: Bool representing if files
        must be pickled or not.
    :param need_ancestors: Bool indicatind if output should.
        contain ancestors of matched term or not.
    """
    run_spacy(
        data_dir=data_dir,
        settings_file=settings_file,
        linker=linker,
        to_pickle=pickle_files,
        need_ancestors=need_ancestors,
    )

    # os.system('say "Done!"')


if __name__ == "__main__":
    freeze_support()
    main()



"""Biohub convertor."""
import logging

EXCLUDE = ["biolink:Publication"]


def parse(input_filename, output_filename) -> None:
    """
    Parse the typical KGX tsv nodes into Bio Term Hub format for compatibility with OGER.

    Mapping of columns from KGX format to Bio Term Hub format,
    -   [0] 'CUI-less' -> UMLS CUI
    -   [1] 'N/A' -> resource from which it comes
    -   [2] CURIE -> native ID
    -   [3] name -> term (this is the field that is tokenized)
    -   [4] name -> preferred form
    -   [5] category -> type

    :param input_filename: (str) Input file path.
    :param output_filename: (str) Output file path.
    :return: None.
    """
    counter = 0
    outstream = open(output_filename, "w+")
    header_dict = None

    with open(input_filename) as fh:
        for line in fh:
            if counter == 0:
                header = line.rstrip().split("\t")
                header_dict = parse_header(header)
                counter += 1
                continue

            elements = [x.rstrip() for x in line.split("\t")]
            if any(x in elements[header_dict["category"]] for x in EXCLUDE):
                # 'category' field is one of the ones in EXCLUDE list
                logging.info(
                    f"Skipping line as part \
                               of excludes: {line.rstrip()}"
                )
                continue

            if not elements[header_dict["name"]]:
                # no 'name' field for record
                print(
                    f"Skipping line as it does not \
                      have a name field: {line.rstrip()}"
                )
                continue

            parsed_record = list()
            parsed_record.append("CUI-less")
            if "provided_by" in header_dict:
                parsed_record.append(elements[header_dict["provided_by"]])
            else:
                parsed_record.append("N/A")
            parsed_record.append(elements[header_dict["id"]])
            parsed_record.append(elements[header_dict["name"]])
            parsed_record.append(elements[header_dict["name"]])
            parsed_record.append(elements[header_dict["category"]])
            if elements[header_dict["synonym"]]:
                synonyms = elements[header_dict["synonym"]]
                for s in synonyms.split("|"):
                    syn_record = [x for x in parsed_record]
                    syn_record[3] = s
                    if syn_record[3].lower() != syn_record[4].lower():
                        syn_record[2] = syn_record[2] + "_SYNONYM"
                        # Preferred form == Synonym matched
                        syn_record[4] = (
                            syn_record[3] + "[SYNONYM_OF:" + syn_record[4] + "]"
                        )
                        write_line(syn_record, outstream)
            write_line(parsed_record, outstream)


def parse_header(elements) -> dict:
    """
    Parse headers from nodes TSV.

    :param elements:The header record.
    :type elements:List
    :return:A dictionary of node header names to index.
    :rtype:Dict
    """
    header_dict = {}
    for col in elements:
        header_dict[col] = elements.index(col)
    return header_dict


def write_line(elements, outstream) -> None:
    """
    Write line to OUTSTREAM.

    :param elements: list - The record to write.
    :param OUTSTREAM: File handle to the output file.
    :return: None.
    """
    outstream.write("\t".join(elements) + "\n")



"""Run OGER."""
import configparser
import csv
import os

import click
from oger.ctrl.router import PipelineServer, Router
from oger.ctrl.run import run
from oger.doc import EXPORTERS

from ontorunner.post import NODE_AND_EDGE_DIR, add_sentence


def run_oger(
    content="data/input",
    termlist="data/terms/DICT.tsv",
    output="data/output/",
    output_format="tsv",
    settings="settings.ini",
    workers=1,
    nodes_and_edges=NODE_AND_EDGE_DIR,
    need_ancestors=True,
) -> None:
    """Run OGER.

    :param content: Input file OR folder containing txt files.
    :param termlist: Path to the dictionary (TSV format).
    :param output: Path to save the output file.
    :param output_format: tsv (default).
    :param settings: If this is provided, all other arguments
    are provided in this file and are hence optional.
    Make changes to this file according to project needs
    s(default:'settings.ini').
    :param workers: Number of parallel threads (default = 1).
    :param nodes_and_edges: Directory where KGX nodes and edges tsv files.
    :param need_ancestors: Bool to decide if ancestors should be present in
    the output or no.
    :return: None.
    """
    if settings:
        config = configparser.ConfigParser()
        config.read(settings)
        sections = config._sections
        settings = sections["Main"]
        settings["n_workers"] = workers
        output = settings["output-directory"]
        input = settings["input-directory"]
        nodes_and_edges = nodes_and_edges
        run(**settings)

    else:
        sniffer = csv.Sniffer()
        sample_bytes = 128
        dialect = sniffer.sniff(open(content).readline(sample_bytes))
        delim = ""
        if dialect.delimiter == "\t" or dialect.delimiter == ",":
            delim = "txt_tsv"
        else:
            delim = "txt"

        conf = Router(termlist_path=termlist, include_header=True)
        pl = PipelineServer(conf)
        doc = pl.load_one(content, delim)
        pl.process(doc)
        n = len([x for x in doc.iter_entities()])
        print(f"Number of recognized entities: {n}")
        with open(output, "w", encoding="utf8") as f:
            pl.write(doc, output_format, f)
        # Add sentence
        if os.path.isdir(content):
            input = content
        elif os.path.isfile(content):
            input = os.path.dirname(content)
            output = os.path.dirname(output)

    add_sentence.parse(input, output, nodes_and_edges, need_ancestors)


# os.system('say "Done!"')


@click.group()
def cli():
    """
    Run the OGER module.

    e.g. python -m ontorunner.oger_module run-oger -s ontorunner/settings.ini
    """
    pass


@cli.command("run-oger")
@click.option("--content", "-c", type=click.Path(exists=True))
@click.option("--termlist", "-t", type=click.Path(exists=True))
@click.option("--output", "-o", type=str)
@click.option(
    "--output-format", "-f", type=click.Choice(EXPORTERS), default="bioc_json"
)
@click.option("--settings", "-s", type=click.Path(exists=True))
@click.option("--workers", "-w", default=1)
@click.option(
    "--nodes-and-edges",
    "-n",
    type=click.Path(exists=True),
    default=NODE_AND_EDGE_DIR,
)
@click.option("--need-ancestors", "-a", type=bool, default=True)
def run_oger_click(
    content,
    termlist,
    output,
    output_format,
    settings,
    workers,
    nodes_and_edges,
    need_ancestors,
):
    """
    Run OGER module using CLI.

    :param content: Input file for NER.
    :param termlist: Filepath for termlist TSV file.
    :param output: Directory fro output.
    :param output_format: Output format [default: tsv].
    :param settings: Filepath for settings.ini file.
    :param workers: Number of parallel threads.
    :param nodes_and_edges: Directory where all KGX
        nodes and edges tsv files reside.
    :param need_ancestors: Bool indicating where output
        should contain ancestors of matched term should be present.
    """
    run_oger(
        content,
        termlist,
        output,
        output_format,
        settings,
        workers,
        nodes_and_edges,
        need_ancestors,
    )


if __name__ == "__main__":
    __spec__ = None
    cli()



"""Add sentences for understanding the context of matched terms."""
import csv
import os
import re
from glob import glob
from typing import List

import nltk
import pandas as pd
import textdistance
from nltk import ne_chunk, pos_tag, word_tokenize
from nltk.stem.wordnet import WordNetLemmatizer

from ontorunner.post.util import (consolidate_rows, filter_synonyms,
                                  get_ancestors, get_column_doc_ratio)

# if not nltk.find("corpora/wordnet"):
nltk.download("wordnet")
nltk.download("punkt")
nltk.download("omw-1.4")
nltk.download("averaged_perceptron_tagger")  # for GH Actions.
nltk.download("maxent_ne_chunker")
nltk.download("words")


pd.options.mode.chained_assignment = None  # default='warn'


def find_extensions(dr, ext) -> List[str]:
    """Find files with a specific extension.

    :param dr: Directory path.
    :param ext: Extension.
    :return: List of relevant files.
    """
    return glob(os.path.join(dr, "*.{}".format(ext)))


def sentencify(input_df, output_df, output_fn):
    """
    Add relevant sentences to the tokenized term in every row of a pandas DataFrame.

    :param df: (DataFrame) pandas DataFrame.
    :return: None
    """
    for _, row in input_df.iterrows():
        idx = row.id
        text = row.text
        # Check for text = NaN
        if text == text:
            text = (
                text.replace("\t", " ")
                .replace("\u2028", " ")
                .replace("\n", "")
                .replace(
                    "\r",
                    "",
                )
                .replace(
                    ")", " "
                )  # reason: re.error: unbalanced parenthesis at position x
                .replace("(", " ")
            )
            sent_tok = nltk.sent_tokenize(text)
            # ** This block is for freq count of term in text
            # # Calcualte frquencies of words and phrases
            # word_tok = nltk.word_tokenize(text.lower())
            # # This gets freq count for each word
            # word_tok_freq = FreqDist(word_tok)
            # # This will capture phrases and corresponding frequencies
            # phrases = dict()
            # for size in 2, 3, 4, 5:
            #     phrases[size] = FreqDist(ngrams(word_tok, size))

            sub_df = output_df[output_df["document_id"] == idx]
            # In certain instances, in spite of the 'matched' and 'preferred'
            # terms being the same, the term is registered as a synonym by KGX
            # and hence the biohub_converter codes this with a '_SYNONYM' tag.
            # In order to counter this, we need to filter these extra rows out.
            if not sub_df.empty and any(sub_df["object_id"].str.endswith("_SYNONYM")):
                sub_df = filter_synonyms(sub_df)

            if len(sent_tok) == 1:
                sub_df["sentence"] = text
            else:
                relevant_sents = []

                for i, row2 in sub_df.iterrows():
                    start_reached = False
                    end_reached = False
                    term_of_interest = str(row2["matched_term"])
                    start_pos = int(row2["start_position"])
                    if start_pos == 0:
                        start_reached = True
                    end_pos = int(row2["end_position"])
                    if end_pos == len(text):
                        end_reached = True
                    if term_of_interest == "nan":
                        # get just the portion where
                        # 'matched_term' == 'preferred_form'
                        # because in case of _SYNONYM,
                        # there will be extra metadata which
                        # will not be present in the tokenized sentence.
                        term_of_interest = str(row2["preferred_form"])

                    relevant_sents = [
                        x
                        for x in sent_tok
                        if re.search(term_of_interest, x, re.IGNORECASE)
                    ]

                    if relevant_sents == []:
                        relevant_sents = ["irrelevant token: can be ignored."]

                    list_of_sents = relevant_sents
                    count = 0

                    while len(list_of_sents) > 1:
                        count += 1
                        # This keeps track of the # of times the start_pos
                        # and/or end_pos are shifted
                        # Detect the beginning and ending of sentences
                        # ---------------------
                        if not start_reached:
                            start_pos -= 1
                            for sent in list_of_sents:
                                if sent.startswith(text[start_pos:end_pos]):
                                    start_reached = True
                                elif text[start_pos:end_pos].startswith("."):
                                    start_pos += 1
                                    start_reached = True

                        if not end_reached:
                            end_pos += 1
                            for sent in list_of_sents:
                                if sent.endswith(text[start_pos:end_pos]):
                                    end_reached = True
                                elif text[start_pos:end_pos].endswith("."):
                                    end_pos -= 1
                                    end_reached = True
                        # -------------------------------------------------------------------
                        term_of_interest = text[start_pos:end_pos]
                        print(
                            f"Doc: {row.id} |"
                            f" iter: {i} |"
                            f" count: {count} |"
                            f" #_of_sent: {len(list_of_sents)}"
                            f" search_term: {term_of_interest}"
                        )

                        list_of_sents = [
                            x
                            for x in relevant_sents
                            if re.search(term_of_interest.strip(), x)
                        ]

                        if count > 30 and 1 < len(list_of_sents):
                            list_of_sents = [list_of_sents[0]]
                            count = 0
                            break
                        # Reason for the break:
                        # In some instance the sentences are repeated.
                        # In such cases the expanding window with start_pos and
                        # end_pos goes expanding after 30 character match
                        # (arbitrary) we take the first element out of the
                        # common terms and take that as the SENTENCE
                        # and then 'break'-ing out of the 'while' loop.
                        # Else, it'll continue looking for the unique
                        # sentence forever. It's a hack but for now it'll
                        # do until severe consequences detected.

                    sub_df.loc[i, "sentence"] = list_of_sents[0]

            if not sub_df.empty:
                sub_df["object_sentence_%"] = sub_df.apply(
                    lambda x: 1
                    - textdistance.jaccard.distance(
                        x.matched_term.lower(), x.sentence.lower()
                    ),
                    axis=1,
                )

                sub_df.to_csv(output_fn, mode="a", sep="\t", header=None, index=None)


def get_match_type(token1: str, token2: str) -> str:
    """
    Return type of token match.

    :param token1: token from 'matched_term'
    :type token1: str
    :param token2: token from 'preferred_term'
    :type token2: str
    :return: Type of match [e.g.: 'exact_match' etc.]
    :rtype: str
    """
    match = ""
    lemma = WordNetLemmatizer()

    if token1.lower() == token2.lower():
        match = "exact_match"
    elif lemma.lemmatize(token1) == lemma.lemmatize(token2):
        # pos = NOUN by default
        match = "lemmatic_match"
    elif lemma.lemmatize(token1, pos="v") == lemma.lemmatize(token2, pos="v"):
        # testing pos = VERB
        match = "lemmatic_match"
    elif lemma.lemmatize(token1, pos="a") == lemma.lemmatize(token2, pos="a"):
        # testing pos = ADJECTIVE
        match = "lemmatic_match"
    elif lemma.lemmatize(token1, pos="r") == lemma.lemmatize(token2, pos="r"):
        # testing pos = ADVERB
        match = "lemmatic_match"

    return match


def parse(
    input_directory: str,
    output_directory: str,
    nodes_and_edges: str,
    need_ancestors: bool,
) -> None:
    """
    Parse OGER output and add sentences of tokenized terms.

    :param input_directory: (str) Input directory path.
    :param output_directory: (str) Output directory path.
    :param nodes_and_edges: (str) Nodes and edges file directory path.
    :return: None.
    """
    # Get a list of potential input files for particular formats
    input_list_tsv = find_extensions(input_directory, "tsv")
    input_list_txt = find_extensions(input_directory, "txt")
    output_files = find_extensions(output_directory, "tsv")
    output_file_list = [
        x
        for x in output_files
        if "_node" not in x
        if "_edge" not in x
        if "ontoRunNER" not in x
    ]
    for output_file in output_file_list:
        output_df = pd.read_csv(output_file, sep="\t", low_memory=False)
        output_df.columns = output_df.columns.str.replace(" ", "_").str.lower()
        # Consolidate rows where the entitys is the same
        # and recognized from multiple origins
        output_df = output_df.rename(
            columns={"entity_id": "object_id", "type": "object_category"}
        )

        output_df = consolidate_rows(output_df)

        output_df[["preferred_form", "object_label"]] = output_df[
            "preferred_form"
        ].str.split("\\[SYNONYM_OF:", expand=True)

        output_df["object_label"] = output_df["object_label"].str.replace(
            "]", "", regex=True
        )
        output_df["object_label"] = output_df["object_label"].fillna(
            output_df["preferred_form"]
        )

        output_df["pos_and_ne_chunk"] = (
            output_df["matched_term"]
            .apply(word_tokenize)
            .apply(pos_tag)
            .apply(ne_chunk)
        )

        output_df = get_column_doc_ratio(output_df, "object_label")
        output_df = get_column_doc_ratio(output_df, "matched_term")

        # Add column which indicates how close
        # of a match is the recognized entity.
        output_df.insert(
            6,
            "match_type",
            output_df.apply(
                lambda x: get_match_type(
                    x.matched_term.lower(), x.preferred_form.lower()
                ),
                axis=1,
            ),
        )

        # Levenshtein distances
        output_df.insert(
            7,
            "levenshtein_distance",
            output_df.apply(
                lambda x: textdistance.levenshtein.distance(
                    x.matched_term.lower(), x.preferred_form.lower()
                ),
                axis=1,
            ),
        )

        # Jaccard Index

        output_df.insert(
            8,
            "jaccard_index",
            output_df.apply(
                lambda x: textdistance.jaccard.distance(
                    x.matched_term.lower(), x.preferred_form.lower()
                ),
                axis=1,
            ),
        )

        # Monge-Elkan
        output_df.insert(
            9,
            "monge_elkan",
            output_df.apply(
                lambda x: textdistance.monge_elkan.distance(
                    x.matched_term.lower(), x.preferred_form.lower()
                ),
                axis=1,
            ),
        )

        output_df["sentence"] = ""

        output_df["object_sentence_%"] = ""

        output_df = output_df.reindex(
            columns=[
                "document_id",
                "object_category",
                "start_position",
                "end_position",
                "matched_term",
                "preferred_form",
                "object_label",
                "object_label_doc_ratio",
                "matched_term_doc_ratio",
                "match_type",
                "levenshtein_distance",
                "jaccard_index",
                "monge_elkan",
                "object_id",
                "pos_and_ne_chunk",
                "sentence_id",
                "umls_cui",
                "origin",
                "sentence",
                "object_sentence_%",
            ]
        )
        # TODO: Maybe use OAK for getting ancestors (?)
        if need_ancestors:
            output_df = get_ancestors(output_df, nodes_and_edges)

        final_output_file = output_file.replace(".tsv", "_ontoRunNER.tsv")

        pd.DataFrame(output_df.columns).T.to_csv(
            final_output_file, sep="\t", index=None, header=None
        )

        if len(input_list_tsv) > 0:
            for f in input_list_tsv:
                input_df = pd.read_csv(f, sep="\t", low_memory=False, index_col=None)
                sentencify(input_df, output_df, final_output_file)

        if len(input_list_txt) > 0:
            # Read each text file such that Id = filename and text = full text
            for f in input_list_txt:
                input_df = pd.DataFrame(columns=["id", "text"])
                sniffer = csv.Sniffer()
                sample_bytes = 128
                dialect = sniffer.sniff(open(f).readline(sample_bytes))
                if dialect.delimiter == "\t" or dialect.delimiter == ",":
                    input_df = pd.read_csv(
                        f, sep="\t", low_memory=False, index_col=None
                    )
                else:
                    id = f.split("/")[-1].split(".txt")[0]
                    with open(f, "r") as fn:
                        text = fn.readlines()
                        text = "".join(text).replace("\n", " ")
                    input_df = input_df.append(
                        {"id": id, "text": text}, ignore_index=True
                    )

                sentencify(input_df, output_df, final_output_file)



"""Constants."""

import configparser
import re
from os import pardir
from os.path import abspath, dirname, join
from pathlib import Path

__version__ = "0.1.0"
# import pathlib

# HERE = pathlib.Path(__file__).parent.resolve()

DATA_DIR_NAME = "data"
TERMS_DIR_NAME = "terms"
SERIAL_DIR_NAME = "serialized"
INPUT_DIR_NAME = "input"
OUTPUT_DIR_NAME = "output"

PARENT_DIR = abspath(join(dirname(__file__), pardir))
DATA_DIR = join(PARENT_DIR, DATA_DIR_NAME)
ONTO_TERMS_FILENAME = "onto_termlist.tsv"
TERMS_PICKLED_FILENAME = "terms.pickle"
PATTERN_LIST_PICKLED_FILENAME = "patterns.pickle"
PHRASE_MATCHER_PICKLED_FILENAME = "phrase_matcher.pickle"
SETTINGS_FILENAME = "settings.ini"
STOPWORDS_FILENAME = "stopWords.txt"

SETTINGS_FILE_PATH = join(dirname(__file__), SETTINGS_FILENAME)


def _get_config(param: str, settings_file: Path = SETTINGS_FILE_PATH):
    read_config = configparser.ConfigParser()
    read_config.read(settings_file)
    main_section = dict(read_config.items("Main"))
    if param == "termlist":
        return [
            v.split("/")[-1]
            for k, v in main_section.items()
            if param in k and re.search(r"\d", k)
        ]
    else:
        return [v.replace("data/", "") for k, v in main_section.items() if param in k]



"""Constants."""

import os

PARENT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
DATA_DIR = os.path.join(PARENT_DIR, "data")
SERIAL_DIR = os.path.join(DATA_DIR, "serialized")



"""Utility functions called after NER."""
import os
from typing import List

import numpy as np
import pandas as pd

from . import NODE_AND_EDGE_DIR, SUBCLASS_PREDICATE


def filter_synonyms(df: pd.DataFrame) -> pd.DataFrame:
    """
    Consolidate entities where '_SYNONYM' object_id is a duplicate.

    :param df: Input DataFrame
    :type df: pd.DataFrame
    :return: Consolidated Dataframe
    :rtype: pd.DataFrame
    """
    condition_1 = df["matched_term"].str.lower() == df["preferred_form"].str.lower()
    condition_2 = df["object_id"].str.contains("_SYNONYM")
    same_yet_syn_condition = condition_1 & condition_2
    new_df = df[~same_yet_syn_condition]
    tmp_df = df[same_yet_syn_condition]
    tmp_df["object_id"] = tmp_df["object_id"].str.replace("_SYNONYM", "")
    new_df = pd.concat([new_df, tmp_df])
    return new_df


def consolidate_rows(df: pd.DataFrame) -> pd.DataFrame:
    """
    Group rows by all columns except "origin".

    This is done to remove redundancies
    created by entity recognition from multiple sources/ontologies

    :param df: Input DataFrame
    :type df: pd.DataFrame
    :return: Consolidated DataFrame
    :rtype: pd.DataFrame
    """
    # drops columns where all rows are None
    df.dropna(axis=1, how="all", inplace=True)
    grouping_columns = df.columns.tolist()
    grouping_columns.remove("origin")

    new_df = (
        df.fillna("tmp")
        .groupby(grouping_columns)
        .agg({"origin": lambda o: " | ".join(o)})
        .reset_index()
        .replace({"object_match_field": {"tmp": ""}})
    )
    return new_df


def get_column_doc_ratio(df: pd.DataFrame, column: str) -> pd.DataFrame:
    """Get str to document ratio of given column in a pandas DataFrame.

    :param df: Pandas DataFrame
    :param column: Column name of the term
    :return: Pandas DataFrame with additional
            columns showing term:document ratio
    """
    df[column] = df[column].str.lower()
    doc_label_df = df[["document_id", column]].drop_duplicates()

    total_docs = len(df["document_id"].drop_duplicates())

    doc_count = column + "_doc_count"

    doc_count_df = (
        doc_label_df.groupby(column)
        .size()
        .sort_values(ascending=False)
        .reset_index(name=doc_count)
    )
    # This new column calculates the ratio:
    # (# of documents where the str in 'column' appears) / (Total # of docs)
    new_column = column + "_doc_ratio"

    doc_count_df[new_column] = doc_count_df[doc_count] / total_docs

    df = df.merge(doc_count_df, how="left", on=column)
    df = df.loc[df.astype(str).drop_duplicates().index]
    return df


def ancestor_generator(df: pd.DataFrame, obj_series: pd.DataFrame) -> List[str]:
    """
    Return an ancestor list of a CURIE.

    :param df: KGX edges of source ontology in DataFrame form.
    :return: List of CURIES (ancestors)
    """
    ancestor_list = []
    obj_series_df = df.loc[df["subject"] == obj_series.object_id]
    while not df.loc[df["subject"] == obj_series.object_id].empty:
        obj_series_df = df.loc[df["subject"] == obj_series.object_id]
        ancestor_list.append(obj_series_df.iloc[0].object_id)
        obj_series = obj_series_df.iloc[0]

    return ancestor_list


def get_ancestors(
    df: pd.DataFrame, nodes_and_edges_dir: str = NODE_AND_EDGE_DIR
) -> pd.DataFrame:
    """
    Return a DataFrame with 'ancestors' column.

    :param df: Input dataframe containing intermediate NER result.
    :param nodes_and_edges_dir: Dir location of KGX edges & nodes file (tsv)
    :return: Dataframe with an 'ancestors' column.
    """
    df["ancestors"] = ""
    object_origin = df[["object_id", "origin"]]
    object_origin["object_id"] = object_origin["object_id"].str.replace("_SYNONYM", "")
    object_origin = object_origin.drop_duplicates()
    all_origins = object_origin["origin"].drop_duplicates().tolist()
    all_origins = [ogn for ogn in all_origins if "|" not in ogn]

    for o in all_origins:
        object_origin_subset = object_origin.loc[object_origin["origin"] == o]
        ont_name = o.split(".")[0]
        ont_edge_file = os.path.join(nodes_and_edges_dir, ont_name + "_edges.tsv")
        print(f"Getting ancestors for {ont_name} terms....")
        ont_edge_df = pd.read_csv(ont_edge_file, sep="\t", low_memory=False)
        ont_edge_df = ont_edge_df.loc[ont_edge_df["predicate"] == SUBCLASS_PREDICATE]
        ont_edge_df.rename(columns={"object": "object_id"}, inplace=True)
        for idx, obj in object_origin_subset.T.iteritems():
            list_of_ancestors = ancestor_generator(ont_edge_df, pd.Series(obj).T)
            df.loc[
                (df["object_id"] == object_origin_subset["object_id"][idx])
                & (df["origin"] == object_origin_subset["origin"][idx]),
                "ancestors",
            ] = str(list_of_ancestors)

    return df.replace(np.nan, "")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ontorunner',
 'ontorunner.converters',
 'ontorunner.pipes',
 'ontorunner.post',
 'ontorunner.pre']

package_data = \
{'': ['*']}

install_requires = \
['OGER>=1.5,<2.0',
 'click>=8.1.3,<9.0.0',
 'dframcy>=0.1.6,<0.2.0',
 'kgx>=1.5.8,<2.0.0',
 'pandas>=1.4.2,<2.0.0',
 'scispacy==0.5.0',
 'six>=1.16.0,<2.0.0',
 'spacy>=3.2.0,<3.3.0',
 'textdistance[extras]>=4.2.2,<5.0.0']

setup_kwargs = {
    'name': 'ontorunner',
    'version': '0.1.0',
    'description': 'This repository is a wrapper project around various entity recognition (NER) tools.',
    'long_description': '# ontoRunNER\n\nThis repository is a wrapper project around the following named entity recognition (NER) tools:\n - [OGER](https://github.com/OntoGene/OGER).\n - [spaCy](https://spacy.io)\n   - using [sciSpaCy](https://scispacy.apps.allenai.org) pipeline \n   namely the CRAFT corpus (`en_ner_craft_md`) used by default. Others can be used as listed [here](https://github.com/allenai/scispacy#available-models)\n\n## Documentation\n\n[ontoRunNER using Sphinx](https://monarch-initiative.github.io/ontorunner/index.html)\n',
    'author': 'Harshad Hegde',
    'author_email': 'hhegde@lbl.gov',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
